
local addon_name, addon = ...

local event_frame = CreateFrame("FRAME")
event_frame:SetScript("OnEvent", function(frame, event, ...)
    local func = frame[event]
    if func then
        return func(frame, ...)
    end
end)

event_frame.on_addon = {}
local function OnAddOnLoaded(addon, func)
    if IsAddOnLoaded(addon) then
        return func()
    end

    if event_frame.on_addon[addon] ~= nil then
        tinsert(event_frame.on_addon[addon], func)
    else
        event_frame.on_addon[addon] = {func}
    end
end

function event_frame:ADDON_LOADED(addon_name)
    if event_frame.on_addon[addon_name] ~= nil then
        for i, func in ipairs(event_frame.on_addon[addon_name]) do
            func()
        end
        event_frame.on_addon[addon_name] = nil
    end
end
event_frame:RegisterEvent("ADDON_LOADED")

--
-- Mount Collection
--

OnAddOnLoaded("Blizzard_Collections", function()
    hooksecurefunc("MountJournal_UpdateMountDisplay", function()
        MountJournal.MountDisplay.InfoButton.Lore:SetText(descriptionText)
        local spellID = MountJournal.MountDisplay.lastDisplayed
        if spellID then
            addon:TextAppendID_NoNL(MountJournal.MountDisplay.InfoButton.Lore, "Spell", spellID)
        end
    end)
end)


--
-- PetJournal
--

do
    OnAddOnLoaded("Blizzard_Collections", function()
        PetJournalPetCardPetInfo:HookScript("OnEnter", function(self)
            if GameTooltip:GetOwner() == self then
                local speciesID = PetJournalPetCard.speciesID
                local _, _, _, creatureID = C_PetJournal.GetPetInfoBySpeciesID(speciesID)
                addon:AddIDLine(GameTooltip, "Species", speciesID, "NPC", creatureID)
            end
        end)
    end)


    local origSharedPetBattleAbilityTooltip_SetAbility = SharedPetBattleAbilityTooltip_SetAbility
    SharedPetBattleAbilityTooltip_SetAbility = function(tip, abilityInfo, additionalInfo)
        local abilityID = abilityInfo:GetAbilityID()
        if abilityID then
            local line = addon:IDLine("Ability", abilityID)

            if additionalInfo then
                additionaInfo = additionalInfo.."\n"..line
            else
                additionalInfo = line
            end
        end

        return origSharedPetBattleAbilityTooltip_SetAbility(tip, abilityInfo, additionalInfo)
    end
end


--
-- Quest Frame
--

hooksecurefunc("QuestMapFrame_UpdateAll", function()
    for i, frame in ipairs(QuestMapFrame.QuestsFrame.Contents.Titles) do
        if not frame._alltheids_hooked then
            frame._alltheids_hooked = true
            frame:HookScript("OnEnter", function(self)
                local _, _, _, _, _, _, _, questID = GetQuestLogTitle(self.questLogIndex)
                if questID and GameTooltip:GetOwner() == self then
                    addon:AddIDLine(GameTooltip, "Quest", questID)
                end
            end)
        end
    end
end)

--
-- Reputation Frame
--

hooksecurefunc("ReputationFrame_Update", function()
    if ReputationDetailFrame:IsShown() then
        local selected = GetSelectedFaction()
        local _, _, _, _, _, _, _, _, _, _, _, _, _, factionID = GetFactionInfo(selected)
        addon:TextAppendID(ReputationDetailFactionDescription, "Faction", factionID)
    end
end)

--
-- World Map
--

hooksecurefunc("WorldMapFrame_UpdateMap", function()
    local frame = WorldMapDetailFrame
    local display = frame._alltheid_display
    if not frame._alltheid_display then
        display = frame:CreateFontString(nil, "OVERLAY")
        frame._alltheid_display = display
        display:SetFontObject(GameFontHighlightLeft)
        display:SetJustifyH("LEFT")
        display:SetHeight(24)
        display:SetWidth(128)
        display:SetPoint("BOTTOMLEFT")
    end

    display:SetText(addon:IDLine("Map", GetCurrentMapAreaID()))
end)
